#include <stdio.h>
#include <usb.h>
#include <fcntl.h>
#include "node.h"
#include "dstar.h"

#define FALSE   0
#define TRUE    1

usb_dev_handle *udev;
int debugSW;

int	SendData (usb_dev_handle *dev_h, char buf[], int len);

int main(int argc, const char **argv) {
	struct usb_bus *bus;
	struct usb_device *dev;
	struct dv_header *dv;
	int dev_found, ret;
	char buffer[9];
	char radio_header[41];
	char flags[3];
	unsigned char status;
	FILE *voice;

	int	resync;

	int i,k;
	int	HeaderLength;


	debugSW = FALSE;
	if (argc >= 2)
	{
		for (i = 1 ; i < argc ; i++)
		{
			if (!strncmp (argv[i],"-d",2)) 	debugSW = TRUE;
		}
	}



	usb_init();
	usb_find_busses();
	usb_find_devices();


	resync = 0;
	udev = NULL;
	dev_found = FALSE;
	for (bus = usb_get_busses(); bus && !dev_found; bus = bus->next) {
        	for (dev = bus->devices; dev && !dev_found; dev = dev->next) {
            	if ((dev->descriptor.idVendor == 0x04D8) && (dev->descriptor.idProduct == 0x0300)) {
                	dev_found = TRUE;
                	udev = usb_open(dev);
            	}
        	}
	}

	if (!dev_found) {
		printf("No matching device found...\n");
		return;
	}

	ret = usb_set_configuration (udev, 1);
        if (debugSW) printf ("\n* Debug set_configureation %d *\n",ret);

	printf ("\n");
	printf ("DV Echo Test program V00.07 \n(C) 2009 Satoshi Yasuda 7m3tjz/ad6gz\n");
	printf ("Do not distribute this program without my permission!\n");
	printf ("\n");

/* PIC VERSION READ */
	printf ("PIC version: ");
	ret = 8;
	while (ret == 8)
	{
        	ret = usb_control_msg(udev, 0xC0, GET_VERSION, 0, 0, buffer, 8, 100);
		if (debugSW) printf ("\n* Debug GET_VERSION %d * \n",ret);
		for (i = 0 ; i < ret ; i++)
		{
			printf ("%c",buffer[i]);
		} 	
	}
	printf ("\n");


   while (1)
   {
	k = 0;
	HeaderLength = 0;


	while (HeaderLength < 41)
	{
        	ret = usb_control_msg(udev, 0xC0, GET_HEADER, 0, 0, buffer, 8, 300);
		if (debugSW && (ret != 0)) printf ("\n* Debug GET_HEADER %d * \n",ret);
		if (ret < 0) {
			if (ret == -116)
			{
				usleep(10000);
			} else {
				printf("** GET_HEADER ret = %d **\n", ret);
				
				if (ret == -5)
				{
				        usb_close(udev);
					return;
				}
			}
       		} else
		{
			if (ret != 0)
			{
				HeaderLength += ret;
				for (i = 0 ; i < ret; i++)
				{
					radio_header[k] = buffer[i];
					k++;
				}
			} else {
				usleep (10000);
		        	ret = usb_control_msg(udev, 0xC0, GET_AD_STATUS, 0, 0, buffer, 1, 100);
				if (debugSW && (ret != 1)) printf ("\n* Debug GET_AD_STATUS in GET_HEADER %d * \n",ret);
				while (ret == -116)
				{
					usleep (10000);
		       		 	ret = usb_control_msg(udev, 0xC0, GET_AD_STATUS, 0, 0, buffer, 1, 100);
					if (debugSW && (ret != 1)) printf ("\n* Debug GET_AD_STATUS(repeat) in GET_HEADER %d * \n",ret);
				}
				if (ret == -5)
				{
					printf ("** GET_AD_STATUS in GET_HEADER %d **",ret);
				        usb_close(udev);
					return;
				}				
				if (buffer[0] & 0x02)
				{
					k = 0;
					HeaderLength = 0;
				} 
				
			}
		}
	}


	dv = (struct dv_header *) radio_header;

	printf ("\n");

	printf ("Flags  : %2.2x %2.2x %2.2x\n",dv->flags[0],dv->flags[1],dv->flags[2]);
	printf ("RPT2   : %.8s\n",dv->RPT2Call);
	printf ("RPT1   : %.8s\n",dv->RPT1Call);
	printf ("YuCall : %.8s\n",dv->YourCall);
	printf ("MyCall : %.8s\n",dv->MyCall);
	printf ("MyCall2: %.4s\n",dv->MyCall2);
	printf ("CRC    : %2.2x %2.2x\n",dv->CRC[0],dv->CRC[1]);
	ret = usb_control_msg(udev, 0xC0, GET_AD_STATUS, 0, 0, &status, 1, 100);
	if (debugSW) printf ("\n* Debug GET_AD_STATUS %d\n",ret);
	if ((status & CRC_ERROR)) printf ("CRC error of RF Header\n");


	voice = fopen ("dstar.voice","wb");	

	while (ret <= 8)
	{
		
        	ret = usb_control_msg(udev, 0xC0, GET_DATA, 0, 0, buffer, 8, 100);
		if (debugSW) printf ("\n* Debug GET_DATA %d * \n",ret);
		if (ret != 0)
		{
			for (i = 0 ; i < ret; i++)
			{
				fputc (buffer[i], voice);
			}
		}
		if (ret == 0)
		{
	        	ret = usb_control_msg(udev, 0xC0, GET_AD_STATUS, 0, 0, buffer, 1, 100);
			if (debugSW) printf ("\n* Debug GET_AD_STATUS %d *\n",ret);
			if ((buffer[0] & LastFrameRead)) break;
		}
		if (ret < 0) usleep(20000);	
	}

	fclose (voice);

	sleep (1);

	voice = fopen ("dstar.voice","rb");

	/* Call Sign set */
        usb_control_msg(udev, 0x40, SET_MyCALL, 0, 0, "7M3TJZ E", 8, 100);
	/* change your callsign */
        usb_control_msg(udev, 0x40, SET_MyCALL2, 0, 0, "NODE", 4, 500);
        usb_control_msg(udev, 0x40, SET_YourCALL, 0, 0, dv->MyCall, 8, 100);
        usb_control_msg(udev, 0x40, SET_RPT2CALL, 0, 0, "DIRECT  ", 8, 100);
        usb_control_msg(udev, 0x40, SET_RPT1CALL, 0, 0, "DIRECT  ", 8, 100);
	flags[0] = 0x00;	/* set for repeater */
	flags[1] = 0x00;
	flags[2] = 0x00;
        usb_control_msg(udev, 0x40, SET_FLAGS, 0, 0, flags, 3, 100);

	/* delay time TxDelay  nn x 10mSec. */
//        usb_control_msg(udev, 0x40, SET_DelayTime, 50, 0,buffer, 0, 100);

	/* Check COS */
	usb_control_msg(udev, 0xC0, GET_AD_STATUS, 0, 0, buffer, 1, 100);
	while (buffer[0] & COS_OnOff)		/* until COS OFF */
	{
		 usb_control_msg(udev, 0xC0, GET_AD_STATUS, 0, 0, buffer, 1, 100);
	}
	

	/* PTT ON */
        usb_control_msg(udev, 0x40, SET_PTT, ON, 0, buffer, 0, 100);

	/* Message send your call */
	resync = CallSignPlay (dv->MyCall, resync);
	if (resync < 0) return;

	/* Reply Message send */
	resync = ReplyMsgPlay (resync);
	if (resync < 0) return;

	/* padding for Null Voice data */
	resync = NullVoicePlay (resync);

	/* Send Recieved Voice & Data STREAM */
	k = 0;
	while ((i = fgetc (voice)) != EOF)
	{
		buffer[k] = i;
		k++;
		if (k == 8)
		{
			k = 0;
			if (SendData (udev, buffer, 8)) return;
		}
	}

	if (k > 0)
	{
			SendData (udev, buffer, k);
	}

	/* PTT OFF */
        usb_control_msg(udev, 0x40, SET_PTT, OFF, 0, buffer, 0, 100);

	fclose (voice);

	status = PTT_OnOff;
	while (status & PTT_OnOff)
	{
		usb_control_msg(udev, 0xC0, GET_AD_STATUS, 0, 0, &status , 1, 200);
	}


//	usleep (20000);

  }		/* end while (1) */

/*
        usb_close(udev);
*/


}

int	CallSignPlay (unsigned char CallSign[], int resync)
{
	FILE	*wave;
	char	file_name[] = {"./audio/ .bin"};
	char 	buff[12];
	int	i,k,l,ret;
	char	status;

	for (i = 0 ; i < 6 ; i++)
	{
		if (CallSign[i] != 0x20)
		{
			file_name[8] = tolower(CallSign[i]);
			wave = fopen (file_name,"rb");
			l = 0;
			while ((k = fgetc (wave)) != EOF)
			{
				buff[l] = k;
				l++;			
				if (l == 12)
				{
					l = 0;
					if (resync == 0)
					{
						buff[ 9] = 0x55;
						buff[10] = 0x2d;
						buff[11] = 0x16;
					} else {
						buff[ 9] = 0x70 ^ 0x66;
						buff[10] = 0x4f ^ 0x66;
						buff[11] = 0x93 ^ 0x66;
					}
					resync++;
					if (resync == 21) resync = 0;
					if (SendData(udev, buff, 12)) return -1;
				}
			}

			fclose (wave);
		}
	}

	return resync;
	
}

int	ReplyMsgPlay (int resync)
{
	FILE	*reply;
	char	file_name[] = {"./audio/replymsg.bin"};
	char 	buff[12];
	int	k,l,ret;
	char	status;

	reply = fopen (file_name,"rb");
	l = 0;

	while ((k = fgetc (reply)) != EOF)
	{

		buff[l] = k;

		l++;			
		if (l == 12)
		{
			 l = 0;
			if (resync == 0)
			{
				buff[ 9] = 0x55;
				buff[10] = 0x2d;
				buff[11] = 0x16;
			} else {
				buff[ 9] = 0x70 ^ 0x66;
				buff[10] = 0x4f ^ 0x66;
				buff[11] = 0x93 ^ 0x66;
			}
			resync++;
			if (resync == 21) resync = 0;
			if (SendData(udev, buff, 12)) return -1;
		}
	}

	fclose (reply);
	
	return resync;	
}

int	NullVoicePlay (int resync)
{
	int	ret;
	char 	NullVoice[12] = {0x9e,0x8d,0x32,0x88,0x26,0x1a,0x3f,0x61,0xe8,0x16,0x29,0xf5};

	while (resync != 0)
	{
		if (SendData(udev, NullVoice, 12)) return -1;
		resync++;
		if (resync == 21) resync = 0;
	}
	
	return resync;	
}
int   SendData(usb_dev_handle *dev_h, char buf[], int len)
{
/*
        dev_h: device handler of Node Adapter
        buf:    sending data in this (array)
        len:    size of buf

        return code
                 0: normal
                -1: Node Adapter is setting PTT OFF.
                        (no more data in TX buffer)
*/
        int     remain_length;
        int     i, k;
        unsigned char   RemainLength;
        unsigned char   status;

        if (debugSW) printf ("\n* DEBUG * in SendDATA length* %d\n",len);

        if (len <= 0) return;   /* if transfer length is 0 or minus, skip this */

/* Checkthe enough space in TX buffer */
        usb_control_msg (dev_h, 0xc0, GET_REMAINSPACE, 0, 0, &RemainLength, 1, 100);
        if (debugSW) printf ("\n* DEBUG remaining space * %d\n",RemainLength);


/* wait unitl enough space in TX buffer */
        while (RemainLength < len)
        {
                usleep (100000);        /* 100 m sec. wait */
                usb_control_msg (dev_h, 0xc0, GET_REMAINSPACE, 0, 0, &RemainLength, 1, 100);
                if (debugSW) printf ("\n* DEBUG remaining space (in wait loop) * %d\n",RemainLength);
        }

/* Put_data to Node Adapter */
        for (i = 0 ; len > i ; i += 8)
        {
                if ((len-i) < 8) k = len - i;
                else k = 8;
                usb_control_msg (dev_h, 0x40, PUT_DATA, 0, 0, &buf[i], k, 100);
        }

/* Check PTT STATUS */

                usb_control_msg(dev_h, 0xC0, GET_AD_STATUS, 0, 0, &status , 1, 100);
                if (!(status & PTT_OnOff))
                {
                        printf ("Please run on high or realtime mode!\n");
                        return -1;
                }

        return 0;
}

